%% viewmtx example
%  Explore code from
%
% <html>
% <a href="my_viewmtx.html"><code>viewmtx.m</code></a>
% </html>
 

%%
% Define vector that draws a cube

close all
clear
%          1      2      3      4      5      6      7      8
vcube = [ 0 0 0; 1 0 0; 1 1 0; 0 1 0; 0 0 1; 1 0 1; 1 1 1; 0 1 1]';
idx = [1 2 3 4 1 5 6 7 8 5 6 2 3 7 8 4];
vert(1,:) = vcube(1,:);
vert(2,:) = vcube(2,:);
vert(3,:) = vcube(3,:);

% These vectors trace the edges of a unit cube:
x = vert(1,idx);
y = vert(2,idx);
z = vert(3,idx);


%%
% Transform the cube vectors to the screen and plot the object:

A = viewmtx(-37.5,30,25);
[m,n] = size(x);
x4d = [x(:),y(:),z(:),ones(m*n,1)]';
x2d = A*x4d;
x2 = zeros(m,n); y2 = zeros(m,n);
x2(:) = x2d(1,:)./x2d(4,:);
y2(:) = x2d(2,:)./x2d(4,:);
plot(x2,y2)

%%
% Use plot3 to display cube (same nominal parameters as above)
% save the plot as cube.eps

plot3(x,y,z);
view(-37.5,30);
camproj('perspective');
camva(25);
set(gcf,'Position',[300 200 800 600]);
set(gca,'Position',[0 0 1 1]);
v1 = get(gca,'CameraPosition');
fprintf('camera position: %g %g %g\n',v1);
v2 = get(gca,'CameraTarget');
fprintf('camera target: %g %g %g\n',v2);


%%
% view vs. viewmtx

T = viewmtx(-37.5,30)
view(-37.5,30);
M = view

% look at rotation matrices

R1 = T(1:3,1:3)
R2 = M(1:3,1:3)

fprintf('det(R1) %g det(R2) %g\n',det(R1),det(R2));

t1 = R1'*T(1:3,4)
t2 = R2'*M(1:3,4)

fprintf('camera position: %g %g %g\n',v1);

%%
% now perspective

az = -37.5;
el = 30;
phi = 25;
T = viewmtx(-37.5,30,25)
f = sqrt(2)/2/+tan(phi*pi/360);
P = [ 1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 -1/f 1];
test = inv(P)*T

%%
% work through matrix 

R = T(1:3,1:3);
% Transformation to move origin of object coordinate system to TARGET
pos = campos
O1 = [eye(4,3),[-pos';1]];

R = [R [0 0 0]'; [0 0 0 1]]
T = P*R*O1

el = el*pi/180;
az = az*pi/180;
% default target calculation (in viewmtx)
default_target = 0.5 + sqrt(3)/2*[cos(el)*sin(az);-cos(el)*cos(az);sin(el)]

%%

pts = [v2 1];
pt = T*pts'
x0 = pt(1)/pt(4);
y0 = pt(2)/pt(4);
z0 = pt(3)/pt(4);
fprintf('transformed target %f %f %f\n',x0,y0,z0);
fprintf('focal length %g\n',f);


%%

A = [x' y' z' ones(length(x),1)];
htpts = T*A';
x3 = htpts(1,:)./htpts(4,:);
y3 = htpts(2,:)./htpts(4,:);
z3 = htpts(3,:)./htpts(4,:);
close all
plot(x3,y3,'k');
axis equal

% get points on edge of cube
verts = unique(A,'rows');
%verts = [verts ones(length(verts),1)];
[B ix] = sort(verts(:,3));
B = verts(ix,:);

htpts = T*B';
xpts = htpts(1,:)./htpts(4,:);
ypts = htpts(2,:)./htpts(4,:);
zpts = htpts(3,:)./htpts(4,:);

disp([xpts' ypts' zpts'])

hold on
plot(xpts,ypts,'ko');
axis equal

xo = 0.005;
yo = 0.005;
for n=1:8
    str = sprintf('%d',n);
    text(xpts(n)+xo,ypts(n)+yo,str);
end
hold off

%%

rgb = imread('cube.jpg');
[ny nx nz] = size(rgb);
fprintf('image size %g %g\n',nx,ny);
imshow(rgb)

%%
% cube data obtained from *.eps file

% there are only 12 unique rows
load cube
plot([data(1,1) data(1,3)],[data(1,2) data(1,4)],'r')
hold on
for n=2:15
plot([data(n,1) data(n,3)],[data(n,2) data(n,4)])
end
hold off
axis ij

% get unique points of cube
pt = [data(:,1) data(:,2)];
pts = flipud(unique(pt,'rows'));
% arrange in same order as starting points (xpts, ypts)
ix = [5 7 2 4 6 8 1 3];
spts = pts(ix,:)

for n=1:length(spts)
    str = sprintf('%d',n);
    text(spts(n,1),spts(n,2),str);
end

%%
% look at pixel scale between plot3 pixel values
% and our calculated values

scl = 1000;
cx = polyfit(xpts',spts(:,1)/scl,1)
cy = polyfit(ypts',spts(:,2)/scl,1)

close all
xr = [-0.2 0.2];
plot(xpts,spts(:,1)/1000,'ko',xr,polyval(cx,xr));
grid
xlabel('calculated x-values');
ylabel('x-values from cube.eps file');
figure
xr = [-0.25 0.25];
plot(ypts,spts(:,2)/1000,'ko',xr,polyval(cy,xr));
grid
xlabel('calculated y-values');
ylabel('y-values from cube.eps file');

%%

xfit = polyval(cx,xpts');
yfit = polyval(cy,ypts');
xd = spts(:,1)-xfit*scl;
yd = spts(:,2)-yfit*scl;
plot(xd,yd,'ko')
grid;
axis equal
xlabel('x error (pixels)');
ylabel('y error (pixels)');
dm = 0.2;
for n=1:8
    str = sprintf('%d',n);
    text(xd(n)+dm*rand,yd(n)+dm*rand,str);
end
maxerr = squeeze(max([xd; yd]));
fprintf('maxerror %e\n',maxerr);





